if select(3, UnitClass("player")) ~= 9 then return end -- Only load when playing a warlock
local Whipped = _G.Whipped
local tinsert, sformat = table.insert, string.format
local COMBATLOG_OBJECT_AFFILIATION_MINE = COMBATLOG_OBJECT_AFFILIATION_MINE
local Demons = Whipped.Demons
local CooldownDemon = Demons.CooldownDemon

-- Frames
Whipped.Frame_Cooldown = CreateFrame("StatusBar", nil, Whipped.MasterFrame)
local Frame_Cooldown = Whipped.Frame_Cooldown
local Frame_Cooldown_Icon = CreateFrame("Frame", nil, Frame_Cooldown)
local buffer = 0 -- Buffer for when Doomguard is still active (legendary)

function Whipped:Cooldown_SetupOptions()
	local Cooldown = {
		name = 'Cooldown',
		type = 'group',
		args = {
			Thickness = {
				name = 'Thickness',
				order = 1,
				type = 'range',
				min = 2,
				max = 200,
				softMin = 2,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Cooldown.Thickness = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Cooldown.Thickness end
			},
			Padding = {
				name = 'Padding',
				order = 3,
				type = 'range',
				min = 0,
				max = 200,
				softMin = 0,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Cooldown.Padding = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Cooldown.Padding end
			},
			Show = {
				name = 'Show',
				type = 'toggle',
				order = 4,
				set =
					function(info, input)
						self.db.global.Cooldown.Show = input;
						if input == true then
							Frame_Cooldown:Show()
						else
							Frame_Cooldown:Hide()
						end
						Whipped:UpdateAllFrames()
					end,
				get =
					function(info)
						return self.db.global.Cooldown.Show
					end
			},
			Color = {
				name = 'Color',
				type = 'color',
				order = 5,
				set =
					function(info, r, g, b, a)
						self.db.global.Cooldown.Color.r = r;
						self.db.global.Cooldown.Color.g = g;
						self.db.global.Cooldown.Color.b = b;
						self.db.global.Cooldown.Color.a = a;
						Frame_Cooldown:SetStatusBarColor(r,g,b,a)
					end,
				get =
					function(info)
						local color = self.db.global.Cooldown.Color
						return color.r, color.g, color.b, color.a
					end
				
			},
			Color_Empowered = {
				name = 'Color Empowered',
				type = 'color',
				order = 6,
				set =
					function(info, r, g, b, a)
						self.db.global.Cooldown.Color_Empowered.r = r;
						self.db.global.Cooldown.Color_Empowered.g = g;
						self.db.global.Cooldown.Color_Empowered.b = b;
						self.db.global.Cooldown.Color_Empowered.a = a;
					end,
				get =
					function(info)
						local color = self.db.global.Cooldown.Color_Empowered
						return color.r, color.g, color.b, color.a
					end
				
			},
			Color_Cooldown = {
				name = 'Color Cooldown',
				type = 'color',
				order = 7,
				set =
					function(info, r, g, b, a)
						self.db.global.Cooldown.Color_Cooldown.r = r;
						self.db.global.Cooldown.Color_Cooldown.g = g;
						self.db.global.Cooldown.Color_Cooldown.b = b;
						self.db.global.Cooldown.Color_Cooldown.a = a;
					end,
				get =
					function(info)
						local color = self.db.global.Cooldown.Color_Cooldown
						return color.r, color.g, color.b, color.a
					end
				
			}
		}
	}
	self:AddCategory("Cooldown", Cooldown)
end

function Whipped:Cooldown_OnLoad()
	-- Statusbar
	local color = self.db.global.Cooldown.Color
	Frame_Cooldown:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
	Frame_Cooldown:SetMinMaxValues(0, 25)
	Frame_Cooldown:SetStatusBarColor(color.r, color.g, color.b, color.a)

	-- Icon Texture
	Frame_Cooldown_Icon.texture = Frame_Cooldown_Icon:CreateTexture(nil, "BACKGROUND")
	Frame_Cooldown_Icon.texture:SetTexture("Interface\\Icons\\warlock_summon_doomguard")
	Frame_Cooldown_Icon.texture:SetAllPoints(true)
	
	-- Show / Hide
	if not self.db.global.Cooldown.Show then Frame_Cooldown:Hide(); Frame_Cooldown_Icon:Hide() end
end

function Whipped:Cooldown_OnUpdate()
	if CooldownDemon.Summoned then -- Are Summoned or on Cooldown
		local t = GetTime()
		if CooldownDemon.TTL > 0 then -- Living CooldownDemon
			CooldownDemon.TTL = CooldownDemon.CD - t - 155
			Frame_Cooldown:SetValue(CooldownDemon.TTL)
		else -- Dead CooldownDemon
			if CooldownDemon.TTL < 0 then -- Statement block only runs once every summon, Setup for cooldown
				CooldownDemon.TTL = 0
				CooldownDemon.Alive = 0
				CooldownDemon.CD = CooldownDemon.CD - buffer
				buffer = 0
				local color = self.db.global.Cooldown.Color_Cooldown
				Frame_Cooldown:SetStatusBarColor(color.r, color.g, color.b, color.a)
			end
			local val = (155 - (CooldownDemon.CD - t)) / 6.2 -- (CD(180) - TTL(25)) / TTL(25) = 2.6
			Frame_Cooldown:SetValue(val)
			if val > 25 then -- Statement block only runs once every cast, Check if off cooldown (reset the whole thing)
				CooldownDemon.GUID = 0
				CooldownDemon.Summoned = false
				local color = self.db.global.Cooldown.Color
				Frame_Cooldown:SetStatusBarColor(color.r, color.g, color.b, color.a)
			end
		end
	end
end

function Whipped:Cooldown_UpdateFrames()
	local MasterFrame = self.db.global
	local SoulShards = MasterFrame.SoulShards
	local Imps = MasterFrame.Imps
	local Dreadstalkers = MasterFrame.Dreadstalkers
	local Cooldown = MasterFrame.Cooldown
	local anchorID = MasterFrame.Anchor
	local anchor, Oanchor = Whipped.anchors[anchorID], Whipped:OppositeAnchor(anchorID)
	
	-- S = Statusbar, I = Icon
	local S_anchorFrame, S_anchorHelp, S_x, S_y, I_paddingX, I_paddingY = Whipped.Frame_Dreadstalkers
	local temp = Dreadstalkers
	if not Dreadstalkers.Show then
		if not Imps.Show then
			if SoulShards.Show then
				anchorFrame = Whipped.Frame_SoulShard_1
				temp = SoulShards
			else
				anchorFrame = Whipped.MasterFrame
				temp = MasterFrame
			end
		else
			anchorFrame = Whipped.Frame_Imps_1
			temp = Imps
		end
	end
	
	-- Logic
	if anchorID == 1 or anchorID == 2 then
		I_paddingY = 0
		if anchorID == 1 then
			I_paddingX = MasterFrame.PaddingX
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "TOP", MasterFrame.PaddingX, -(MasterFrame.PaddingY + Cooldown.Padding)
			else
				S_anchorHelp, S_x, S_y = "BOTTOM", 0, -(temp.Padding + Cooldown.Padding)
			end
		else
			I_paddingX = -MasterFrame.PaddingX
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "TOP", -MasterFrame.PaddingX, -(MasterFrame.PaddingY + Cooldown.Padding)
			else
				S_anchorHelp, S_x, S_y = "BOTTOM", 0, -(temp.Padding + Cooldown.Padding)
			end
		end
	else
		I_paddingX = 0
		if anchorID == 3 then
			I_paddingY = -MasterFrame.PaddingY
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "LEFT", (MasterFrame.PaddingX + Cooldown.Padding), -MasterFrame.PaddingY
			else
				S_anchorHelp, S_x, S_y = "RIGHT", (temp.Padding + Cooldown.Padding), 0
			end
		else
			I_paddingY = MasterFrame.PaddingY
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "LEFT", (MasterFrame.PaddingX + Dreadstalker.Padding), -MasterFrame.PaddingY
			else
				S_anchorHelp, S_x, S_y = "RIGHT", (temp.Padding + Cooldown.Padding), 0
			end
		end
	end
	-- Frame_Cooldown
	Frame_Cooldown:ClearAllPoints()
	if anchorID == 1 or anchorID == 2 then
		Frame_Cooldown:SetWidth((SoulShards.Width * 5 + SoulShards.Padding * 4))
		Frame_Cooldown:SetHeight(Cooldown.Thickness)
		Frame_Cooldown:SetPoint("TOP"..anchor, S_anchorFrame, S_anchorHelp..anchor, S_x, S_y)
		Frame_Cooldown:SetOrientation("HORIZONTAL")
	else
		Frame_Cooldown:SetWidth(Cooldown.Thickness)
		Frame_Cooldown:SetHeight((SoulShards.Height * 5 + SoulShards.Padding * 4))
		Frame_Cooldown:SetPoint(anchor.."LEFT", S_anchorFrame, anchor..S_anchorHelp, S_x, S_y)
		Frame_Cooldown:SetOrientation("VERTICAL")
	end
	
	-- Frame_Cooldown_Icon
	Frame_Cooldown_Icon:SetWidth(Cooldown.Thickness)
	Frame_Cooldown_Icon:SetHeight(Cooldown.Thickness)
	Frame_Cooldown_Icon:ClearAllPoints()
	Frame_Cooldown_Icon:SetPoint(anchor, Frame_Cooldown, Oanchor, I_paddingX, I_paddingY)
end

function Whipped:Cooldown_CLE(...)
	local event = select(2, ...)
	if event == "SPELL_CAST_SUCCESS" then
		if CooldownDemon.Alive > 0 then
			local color = self.db.global.Cooldown.Color_Empowered
			Frame_Cooldown:SetStatusBarColor(color.r, color.g, color.b, color.a)
		end
	elseif event == "SPELL_AURA_REMOVED" then
		if CooldownDemon.TTL > 0 then
			local color = self.db.global.Cooldown.Color
			Frame_Cooldown:SetStatusBarColor(color.r, color.g, color.b, color.a)
		end
	elseif event == "SPELL_SUMMON" then
		local SpellID = select(12, ...)
		if (SpellID == 60478 or SpellID == 111685) and bit.band(select(6,...), COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then -- Doomguard(cd), Infernal(cd)
			CooldownDemon.Summoned = true
			CooldownDemon.CD = GetTime() + 180
			CooldownDemon.TTL = CooldownDemon.CD - 155
			CooldownDemon.Alive = 1
			CooldownDemon.DE = 1
			CooldownDemon.Name = select(9, ...)
			CooldownDemon.GUID = select(8, ...)
		end
	elseif event == "UNIT_DIED" then
		if select(8,...) == CooldownDemon.GUID then -- Doomguard(cd), Infernal(cd)
			if CooldownDemon.Alive > 0 then -- Infernal actually has a UNIT_DIED event when expired, WHY DOES EVERY DEMON WORK COMPLETELY DIFFERENTLY TO THE LAST ONE, c'mon blizz :(
				CooldownDemon.Alive = 0
				CooldownDemon.TTL = 0
				local color = self.db.global.Cooldown.Color
				Frame_Cooldown:SetStatusBarColor(color.r, color.g, color.b, color.a)
			end
		end
	end
end

function Whipped:Cooldown_Legendary(spawns) -- 'Wilfred's Sigil of Superior Summoning' reduces cooldown of Doomguard and Infernal by 2 seconds for every imps/dreadstalker summoned
	if Whipped.legendary then
		if CooldownDemon.TTL > 0 then
			buffer = buffer + 2 * spawns
		else
			CooldownDemon.CD = CooldownDemon.CD - 2 * spawns
		end
	end
end