if select(3, UnitClass("player")) ~= 9 then return end -- Only load when playing a warlock
local Whipped = _G.Whipped
local tinsert, sformat = table.insert, string.format
local COMBATLOG_OBJECT_AFFILIATION_MINE = COMBATLOG_OBJECT_AFFILIATION_MINE
local Demons = Whipped.Demons
local Darkglare = Demons.Darkglare

-- Frames
Whipped.Frame_Darkglare = CreateFrame("StatusBar", nil, Whipped.MasterFrame)
local Frame_Darkglare = Whipped.Frame_Darkglare
local Frame_Darkglare_Icon = CreateFrame("Frame", nil, Frame_Darkglare)

function Whipped:Darkglare_SetupOptions()
	local Darkglare = {
		name = 'Darkglare',
		type = 'group',
		args = {
			Thickness = {
				name = 'Thickness',
				order = 1,
				type = 'range',
				min = 2,
				max = 200,
				softMin = 2,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Darkglare.Thickness = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Darkglare.Thickness end
			},
			Padding = {
				name = 'Padding',
				order = 2,
				type = 'range',
				min = 0,
				max = 200,
				softMin = 0,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Darkglare.Padding = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Darkglare.Padding end
			},
			Show = {
				name = 'Show',
				type = 'toggle',
				order = 4,
				set =
					function(info, input)
						self.db.global.Darkglare.Show = input;
						if input == true then
							if Whipped.Demons.Darkglare.Talented then
								Frame_Darkglare:Show()
							else
								Frame_Darkglare:Hide()
							end
						else
							Frame_Darkglare:Hide()
						end
						Whipped:UpdateAllFrames()
					end,
				get =
					function(info)
						return self.db.global.Darkglare.Show
					end
			},
			Color = {
				name = 'Color',
				type = 'color',
				order = 5,
				set =
					function(info, r, g, b, a)
						self.db.global.Darkglare.Color.r = r;
						self.db.global.Darkglare.Color.g = g;
						self.db.global.Darkglare.Color.b = b;
						self.db.global.Darkglare.Color.a = a;
						Frame_Darkglare:SetStatusBarColor(r,g,b,a)
					end,
				get =
					function(info)
						local color = self.db.global.Darkglare.Color
						return color.r, color.g, color.b, color.a
					end
				
			},
			Color_Empowered = {
				name = 'Color Empowered',
				type = 'color',
				order = 6,
				set =
					function(info, r, g, b, a)
						self.db.global.Darkglare.Color_Empowered.r = r;
						self.db.global.Darkglare.Color_Empowered.g = g;
						self.db.global.Darkglare.Color_Empowered.b = b;
						self.db.global.Darkglare.Color_Empowered.a = a;
					end,
				get =
					function(info)
						local color = self.db.global.Darkglare.Color_Empowered
						return color.r, color.g, color.b, color.a
					end
				
			},
			Color_Cooldown = {
				name = 'Color Cooldown',
				type = 'color',
				order = 7,
				set =
					function(info, r, g, b, a)
						self.db.global.Darkglare.Color_Cooldown.r = r;
						self.db.global.Darkglare.Color_Cooldown.g = g;
						self.db.global.Darkglare.Color_Cooldown.b = b;
						self.db.global.Darkglare.Color_Cooldown.a = a;
					end,
				get =
					function(info)
						local color = self.db.global.Darkglare.Color_Cooldown
						return color.r, color.g, color.b, color.a
					end
				
			}
		}
	}
	self:AddCategory("Darkglare", Darkglare)
end

function Whipped:Darkglare_OnLoad()
	-- Statusbar
	local color = self.db.global.Darkglare.Color
	Frame_Darkglare:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
	Frame_Darkglare:SetMinMaxValues(0, 12)
	Frame_Darkglare:SetStatusBarColor(color.r, color.g, color.b, color.a)

	-- Icon Texture
	Frame_Darkglare_Icon.texture = Frame_Darkglare_Icon:CreateTexture(nil, "BACKGROUND")
	Frame_Darkglare_Icon.texture:SetTexture("Interface\\Icons\\achievement_boss_durumu")
	Frame_Darkglare_Icon.texture:SetAllPoints(true)
	
	-- Show / Hide
	if not self.db.global.Darkglare.Show then Frame_Darkglare:Hide() end
end

function Whipped:Darkglare_OnUpdate()
	if Darkglare.Summoned then -- Are Summoned or on Cooldown
		local t = GetTime()
		if Darkglare.TTL > 0 then -- Living Darkglare
			Darkglare.TTL = Darkglare.CD - t - 12
			Frame_Darkglare:SetValue(Darkglare.TTL)
		else -- Dead Darkglare
			if Darkglare.TTL < 0 then -- Statement block only runs once every summon, Setup for cooldown
				Darkglare.TTL = 0
				Darkglare.Alive = 0
				local color = self.db.global.Darkglare.Color_Cooldown
				Frame_Darkglare:SetStatusBarColor(color.r, color.g, color.b, color.a)
			end
			local val = (12 - (Darkglare.CD - t)) -- (CD(24) - TTL(12)) / TTL(12) = 1
			Frame_Darkglare:SetValue(val)
			if val > 12 then -- Statement block only runs once every cast, Check if off cooldown (reset the whole thing)
				Darkglare.Dead = 0
				Darkglare.GUID = 0
				Darkglare.Summoned = false
				local color = self.db.global.Darkglare.Color
				Frame_Darkglare:SetStatusBarColor(color.r, color.g, color.b, color.a)
			end
		end
	end
end

function Whipped:Darkglare_UpdateFrames()
	local MasterFrame = self.db.global
	local SoulShards = MasterFrame.SoulShards
	local Imps = MasterFrame.Imps
	local Dreadstalkers = MasterFrame.Dreadstalkers
	local Cooldown = MasterFrame.Cooldown
	local Grimoire = MasterFrame.Grimoire
	local Darkglare = MasterFrame.Darkglare
	local anchorID = MasterFrame.Anchor
	local anchor, Oanchor = Whipped.anchors[anchorID], Whipped:OppositeAnchor(anchorID)
	
	-- S = Statusbar, I = Icon
	local S_anchorFrame, S_anchorHelp, S_x, S_y, I_paddingX, I_paddingY = Whipped.Frame_Grimoire
	local temp = Grimoire
	if not Grimoire.Show or not Demons.GrimoireDemon.Talented then
		if not Cooldown.Show then
			if not Dreadstalkers.Show then
				if not Imps.Show then
					if SoulShards.Show then
						S_anchorFrame = Whipped.Frame_SoulShard_1
						temp = SoulShards
					else
						S_anchorFrame = Whipped.MasterFrame
						temp = MasterFrame
					end
				else
					S_anchorFrame = Whipped.Frame_Imps_1
					temp = Imps
				end
			else
				S_anchorFrame = Whipped.Frame_Dreadstalkers
				temp = Dreadstalkers
			end
		else
			S_anchorFrame = Whipped.Frame_Cooldown
			temp = Cooldown
		end
	end
	
	-- Logic
	if anchorID == 1 or anchorID == 2 then
		I_paddingY = 0
		if anchorID == 1 then
			I_paddingX = MasterFrame.PaddingX
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "TOP", MasterFrame.PaddingX, -(MasterFrame.PaddingY + Darkglare.Padding)
			else
				S_anchorHelp, S_x, S_y = "BOTTOM", 0, -(temp.Padding + Darkglare.Padding)
			end
		else
			I_paddingX = -MasterFrame.PaddingX
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "TOP", -MasterFrame.PaddingX, -(MasterFrame.PaddingY + Darkglare.Padding)
			else
				S_anchorHelp, S_x, S_y = "BOTTOM", 0, -(temp.Padding + Darkglare.Padding)
			end
		end
	else
		I_paddingX = 0
		if anchorID == 3 then
			I_paddingY = -MasterFrame.PaddingY
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "LEFT", (MasterFrame.PaddingX + Darkglare.Padding), -MasterFrame.PaddingY
			else
				S_anchorHelp, S_x, S_y = "RIGHT", (temp.Padding + Darkglare.Padding), 0
			end
		else
			I_paddingY = MasterFrame.PaddingY
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "LEFT", (MasterFrame.PaddingX + Dreadstalker.Padding), -MasterFrame.PaddingY
			else
				S_anchorHelp, S_x, S_y = "RIGHT", (temp.Padding + Darkglare.Padding), 0
			end
		end
	end
	
	-- Frame_Darkglare
	Frame_Darkglare:ClearAllPoints()
	if anchorID == 1 or anchorID == 2 then
		Frame_Darkglare:SetWidth((SoulShards.Width * 5 + SoulShards.Padding * 4))
		Frame_Darkglare:SetHeight(Darkglare.Thickness)
		Frame_Darkglare:SetPoint("TOP"..anchor, S_anchorFrame, S_anchorHelp..anchor, S_x, S_y)
		Frame_Darkglare:SetOrientation("HORIZONTAL")
	else
		Frame_Darkglare:SetWidth(Darkglare.Thickness)
		Frame_Darkglare:SetHeight((SoulShards.Height * 5 + SoulShards.Padding * 4))
		Frame_Darkglare:SetPoint(anchor.."LEFT", S_anchorFrame, anchor..S_anchorHelp, S_x, S_y)
		Frame_Darkglare:SetOrientation("VERTICAL")
	end
	
	-- Frame_Darkglare_Icon
	Frame_Darkglare_Icon:SetWidth(Darkglare.Thickness)
	Frame_Darkglare_Icon:SetHeight(Darkglare.Thickness)
	Frame_Darkglare_Icon:ClearAllPoints()
	Frame_Darkglare_Icon:SetPoint(anchor, Frame_Darkglare, Oanchor, I_paddingX, I_paddingY)
end

function Whipped:Darkglare_CLE(...)
	local event = select(2, ...)
	if event == "SPELL_CAST_SUCCESS" then
		if Darkglare.Alive > 0 then
			local color = self.db.global.Darkglare.Color_Empowered
			Frame_Darkglare:SetStatusBarColor(color.r, color.g, color.b, color.a)
		end
	elseif event == "SPELL_SUMMON" then
		local SpellID = select(12, ...)
		if SpellID == 205180 and bit.band(select(6,...), COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then -- Darkglare
			Darkglare.Summoned = true
			Darkglare.CD = GetTime() + 24
			Darkglare.TTL = Darkglare.CD - 12
			Darkglare.Alive = 1
			Darkglare.DE = 1
			Darkglare.GUID = select(8, ...)
		end
	elseif event == "UNIT_DIED" then
		if select(8,...) == Demons.Darkglare.GUID then -- Darkglare
			Darkglare.Alive = 0
			Darkglare.TTL = 0
			local color = self.db.global.Darkglare.Color
			Frame_Darkglare:SetStatusBarColor(color.r, color.g, color.b, color.a)
		end
	end
end

function Whipped:Darkglare_UpdateTalents()
	local id,_,_,b = GetTalentInfo(7,1,1)
	if id == 22478 and b then
		Darkglare.Talented = true
		if self.db.global.Darkglare.Show then
			Frame_Darkglare:Show()
		end
	else
		Darkglare.Talented = false
		Frame_Darkglare:Hide()
	end
end