if select(3, UnitClass("player")) ~= 9 then return end -- Only load when playing a warlock
local Whipped = _G.Whipped
local sformat = string.format -- DEBUG
local COMBATLOG_OBJECT_AFFILIATION_MINE = COMBATLOG_OBJECT_AFFILIATION_MINE
local Demons = Whipped.Demons
local Dreadstalkers = Demons.Dreadstalkers

-- Frames
Whipped.Frame_Dreadstalkers = CreateFrame("StatusBar", nil, Whipped.MasterFrame)
local Frame_Dreadstalkers = Whipped.Frame_Dreadstalkers
local Frame_Dreadstalkers_Icon = CreateFrame("Frame", nil, Frame_Dreadstalkers)
local FontString_Dreadstalkers = Frame_Dreadstalkers_Icon:CreateFontString()

function Whipped:Dreadstalkers_SetupOptions()
	local Dreadstalkers = {
		name = 'Dreadstalkers',
		type = 'group',
		args = {
			Thickness = {
				name = 'Thickness',
				order = 1,
				type = 'range',
				min = 2,
				max = 200,
				softMin = 2,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Dreadstalkers.Thickness = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Dreadstalkers.Thickness end
			},
			Padding = {
				name = 'Padding',
				order = 2,
				type = 'range',
				min = 0,
				max = 200,
				softMin = 0,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Dreadstalkers.Padding = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Dreadstalkers.Padding end
			},
			Show = {
				name = 'Show',
				type = 'toggle',
				order = 3,
				set =
					function(info, input)
						self.db.global.Dreadstalkers.Show = input;
						if input == true then
							Frame_Dreadstalkers:Show()
						else
							Frame_Dreadstalkers:Hide()
						end
						Whipped:UpdateAllFrames()
					end,
				get = function(info) return self.db.global.Dreadstalkers.Show end
			},
			Color = {
				name = 'Color',
				type = 'color',
				order = 4,
				set =
					function(info, r, g, b, a)
						self.db.global.Dreadstalkers.Color.r = r;
						self.db.global.Dreadstalkers.Color.g = g;
						self.db.global.Dreadstalkers.Color.b = b;
						self.db.global.Dreadstalkers.Color.a = a;
						Frame_Dreadstalkers:SetStatusBarColor(r,g,b,a)
					end,
				get =
					function(info)
						local color = self.db.global.Dreadstalkers.Color
						return color.r, color.g, color.b, color.a
					end
				
			},
			Color_Empowered = {
				name = 'Color Empowered',
				type = 'color',
				order = 5,
				set =
					function(info, r, g, b, a)
						self.db.global.Dreadstalkers.Color_Empowered.r = r;
						self.db.global.Dreadstalkers.Color_Empowered.g = g;
						self.db.global.Dreadstalkers.Color_Empowered.b = b;
						self.db.global.Dreadstalkers.Color_Empowered.a = a;
					end,
				get =
					function(info)
						local color = self.db.global.Dreadstalkers.Color_Empowered
						return color.r, color.g, color.b, color.a
					end
				
			},
			Color_Cooldown = {
				name = 'Color Cooldown',
				type = 'color',
				order = 7,
				set =
					function(info, r, g, b, a)
						self.db.global.Dreadstalkers.Color_Cooldown.r = r;
						self.db.global.Dreadstalkers.Color_Cooldown.g = g;
						self.db.global.Dreadstalkers.Color_Cooldown.b = b;
						self.db.global.Dreadstalkers.Color_Cooldown.a = a;
					end,
				get =
					function(info)
						local color = self.db.global.Dreadstalkers.Color_Cooldown
						return color.r, color.g, color.b, color.a
					end
				
			},
			Show_Number = {
				name = 'Show Number',
				type = 'toggle',
				order = 6,
				set =
					function(info, input)
						self.db.global.Dreadstalkers.Show_Number = input;
						if input == true then
							FontString_Dreadstalkers:Show()
						else
							FontString_Dreadstalkers:Hide()
						end
						Whipped:UpdateAllFrames()
					end,
				get =
					function(info)
						return self.db.global.Dreadstalkers.Show_Number
					end
			}
		}
	}
	self:AddCategory("Dreadstalkers", Dreadstalkers)
end

function Whipped:Dreadstalkers_OnLoad()
	-- Statusbar
	local color = self.db.global.Dreadstalkers.Color
	Frame_Dreadstalkers:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
	Frame_Dreadstalkers:SetMinMaxValues(0, 12)
	Frame_Dreadstalkers:SetStatusBarColor(color.r, color.g, color.b, color.a)

	-- Icon Texture
	Frame_Dreadstalkers_Icon.texture = Frame_Dreadstalkers_Icon:CreateTexture(nil, "BACKGROUND")
	Frame_Dreadstalkers_Icon.texture:SetTexture("Interface\\Icons\\Spell_warlock_calldreadstalkers")
	Frame_Dreadstalkers_Icon.texture:SetAllPoints(true)
	
	-- Fontstring
	FontString_Dreadstalkers:SetFont(Whipped.WHIPPED_CHAT_FONT, self.db.global.Dreadstalkers.Thickness / 2)
	FontString_Dreadstalkers:SetTextColor(1,1,1,1)
	FontString_Dreadstalkers:SetText(Whipped.Demons.Dreadstalkers.Alive)
	
	-- Show / Hide
	if not self.db.global.Dreadstalkers.Show then Frame_Dreadstalkers:Hide() end
	if not self.db.global.Dreadstalkers.Show_Number then FontString_Dreadstalkers:Hide() end
end

function Whipped:Dreadstalkers_OnUpdate()
	if Dreadstalkers.Summoned then -- Are Summoned or on Cooldown
		local t = GetTime()
		if Dreadstalkers.TTL > 0 then -- Living Dreadstalkers
			Dreadstalkers.TTL = Dreadstalkers.CD - t - 3
			Frame_Dreadstalkers:SetValue(Dreadstalkers.TTL)
		else -- Dead Dreadstalkers
			if Dreadstalkers.TTL < 0 then -- Statement block only runs once every summon, Setup for cooldown
				Dreadstalkers.TTL = 0
				Dreadstalkers.Alive = 0
				local color = self.db.global.Dreadstalkers.Color_Cooldown
				Frame_Dreadstalkers:SetStatusBarColor(color.r, color.g, color.b, color.a)
				Whipped:ImprovedDreadstalkers()
			end
			local val = (3 - (Dreadstalkers.CD - t)) / .25 -- (CD(15) - TTL(12)) / TTL(12) = .25
			Frame_Dreadstalkers:SetValue(val)
			if val > 12 then -- Statement block only runs once every cast, Check if off cooldown (reset the whole thing)
				Dreadstalkers.Dead = 0
				Dreadstalkers.GUID1 = 0
				Dreadstalkers.Summoned = false
				local color = self.db.global.Dreadstalkers.Color
				Frame_Dreadstalkers:SetStatusBarColor(color.r, color.g, color.b, color.a)
			end
		end
	end
end

function Whipped:Dreadstalkers_UpdateFrames()
	local MasterFrame = self.db.global
	local SoulShards = MasterFrame.SoulShards
	local Imps = MasterFrame.Imps
	local Dreadstalkers = MasterFrame.Dreadstalkers
	local anchorID = MasterFrame.Anchor
	local anchor, Oanchor = Whipped.anchors[anchorID], Whipped:OppositeAnchor(anchorID)
	
	-- S = Statusbar, I = Icon, F = Font
	local S_anchorFrame, S_anchorHelp, S_x, S_y, I_paddingX, I_paddingY, F_paddingX, F_paddingY = Whipped.Frame_Imps_1
	local temp = Imps
	if not Imps.Show then
		if SoulShards.Show then
			S_anchorFrame = Whipped.Frame_SoulShard_1
			temp = SoulShards
		else
			S_anchorFrame = Whipped.MasterFrame
			temp = MasterFrame
		end
	end
	
	-- Logic
	if anchorID == 1 or anchorID == 2 then
		I_paddingY, F_paddingY = 0, 0
		if anchorID == 1 then
			I_paddingX, F_paddingX = MasterFrame.PaddingX, 2
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "TOP", MasterFrame.PaddingX, -(MasterFrame.PaddingY + Dreadstalkers.Padding)
			else
				S_anchorHelp, S_x, S_y = "BOTTOM", 0, -(temp.Padding + Dreadstalkers.Padding)
			end
		else
			I_paddingX, F_paddingX = -MasterFrame.PaddingX, -2
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "TOP", -MasterFrame.PaddingX, -(MasterFrame.PaddingY + Dreadstalkers.Padding)
			else
				S_anchorHelp, S_x, S_y = "BOTTOM", 0, -(temp.Padding + Dreadstalkers.Padding)
			end
		end
	else
		I_paddingX, F_paddingX = 0, 0
		if anchorID == 3 then
			I_paddingY, F_paddingY = -MasterFrame.PaddingY, -2
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "LEFT", (MasterFrame.PaddingX + Dreadstalkers.Padding), -MasterFrame.PaddingY
			else
				S_anchorHelp, S_x, S_y = "RIGHT", (temp.Padding + Dreadstalkers.Padding), 0
			end
		else
			I_paddingY, F_paddingY = MasterFrame.PaddingY, 2
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "LEFT", (MasterFrame.PaddingX + Dreadstalkers.Padding), -MasterFrame.PaddingY
			else
				S_anchorHelp, S_x, S_y = "RIGHT", (temp.Padding + Dreadstalkers.Padding), 0
			end
		end
	end
	
	-- Frame_Dreadstalkers
	Frame_Dreadstalkers:ClearAllPoints()
	if anchorID == 1 or anchorID == 2 then
		Frame_Dreadstalkers:SetWidth((SoulShards.Width * 5 + SoulShards.Padding * 4))
		Frame_Dreadstalkers:SetHeight(Dreadstalkers.Thickness)
		Frame_Dreadstalkers:SetPoint("TOP"..anchor, S_anchorFrame, S_anchorHelp..anchor, S_x, S_y)
		Frame_Dreadstalkers:SetOrientation("HORIZONTAL")
	else
		Frame_Dreadstalkers:SetWidth(Dreadstalkers.Thickness)
		Frame_Dreadstalkers:SetHeight((SoulShards.Height * 5 + SoulShards.Padding * 4))
		Frame_Dreadstalkers:SetPoint(anchor.."LEFT", S_anchorFrame, anchor..S_anchorHelp, S_x, S_y)
		Frame_Dreadstalkers:SetOrientation("VERTICAL")
	end
	
	-- Frame_Dreadstalkers_Icon
	Frame_Dreadstalkers_Icon:SetWidth(Dreadstalkers.Thickness)
	Frame_Dreadstalkers_Icon:SetHeight(Dreadstalkers.Thickness)
	Frame_Dreadstalkers_Icon:ClearAllPoints()
	Frame_Dreadstalkers_Icon:SetPoint(anchor, Frame_Dreadstalkers, Oanchor, I_paddingX, I_paddingY)
		
	-- FontString_Dreadstalkers
	FontString_Dreadstalkers:ClearAllPoints()
	FontString_Dreadstalkers:SetPoint(anchor, Frame_Dreadstalkers_Icon, Oanchor, F_paddingX, F_paddingY)
	FontString_Dreadstalkers:SetFont(Whipped.WHIPPED_CHAT_FONT, Dreadstalkers.Thickness / 2)
end

function Whipped:Dreadstalkers_CLE(...)
	local timestamp, event, playerGUID = select(1, ...), select(2, ...), select(4, ...)
	local SpellName
	if event == "SPELL_CAST_SUCCESS" then
		if Dreadstalkers.Alive > 0 then
			local color = self.db.global.Dreadstalkers.Color_Empowered
			Frame_Dreadstalkers:SetStatusBarColor(color.r, color.g, color.b, color.a)
		end
	elseif event == "SPELL_SUMMON" then
		if (select(12, ...) == 193331 or select(12, ...) == 193332) and bit.band(select(6,...), COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then
			if not Dreadstalkers.Summoned then
				Dreadstalkers.Summoned = true
				Dreadstalkers.CD = GetTime() + 15
				Dreadstalkers.TTL = Dreadstalkers.CD - 3
				Dreadstalkers.Alive = 2
				Dreadstalkers.DE = 1
				Whipped:Cooldown_Legendary(2)
			end
			if Dreadstalkers.GUID1 == 0 then -- Saves Dreadstalker GUIDs to track UNIT_DIED events (UNIT_DIED has no owner data for pets)
				Dreadstalkers.GUID1 = select(8,...)
			else
				Dreadstalkers.GUID2 = select(8,...)
			end
		end
	elseif event == "UNIT_DIED" then
		if select(8,...) == Dreadstalkers.GUID1 or select(8,...) == Dreadstalkers.GUID2 then
			Dreadstalkers.Dead = Dreadstalkers.Dead + 1
			Dreadstalkers.Alive = Dreadstalkers.Alive - 1
			if Dreadstalkers.Alive == 0 then
				local color = self.db.global.Dreadstalkers.Color
				Frame_Dreadstalkers:SetStatusBarColor(color.r, color.g, color.b, color.a)
			end
		end
	elseif event == "SPELL_AURA_APPLIED" then
		if bit.band(select(6,...), COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then
			if select(12,...) ==205146 then -- Demonic Calling
				ActionButton_ShowOverlayGlow(Frame_Dreadstalkers_Icon)
			end
		end
	elseif event == "SPELL_AURA_REMOVED" then
		if bit.band(select(6,...), COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then
			if select(12,...) == 205146 then -- Demonic Calling
				ActionButton_HideOverlayGlow(Frame_Dreadstalkers_Icon)
			end
		end
	end
end


function Whipped:Dreadstalkers_UpdateFontString()
	FontString_Dreadstalkers:SetText(Dreadstalkers.Alive)
end