if select(3, UnitClass("player")) ~= 9 then return end -- Only load when playing a warlock
local Whipped = _G.Whipped
local tinsert, sformat = table.insert, string.format
local COMBATLOG_OBJECT_AFFILIATION_MINE = COMBATLOG_OBJECT_AFFILIATION_MINE
local Demons = Whipped.Demons
local GrimoireDemon = Demons.GrimoireDemon

-- Frames
Whipped.Frame_Grimoire = CreateFrame("StatusBar", nil, Whipped.MasterFrame)
local Frame_Grimoire = Whipped.Frame_Grimoire
local Frame_Grimoire_Icon = CreateFrame("Frame", nil, Frame_Grimoire)

function Whipped:Grimoire_SetupOptions()
	local Grimoire = {
		name = 'Grimoire',
		type = 'group',
		args = {
			Thickness = {
				name = 'Thickness',
				order = 1,
				type = 'range',
				min = 2,
				max = 200,
				softMin = 2,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Grimoire.Thickness = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Grimoire.Thickness end
			},
			Padding = {
				name = 'Padding',
				order = 3,
				type = 'range',
				min = 0,
				max = 200,
				softMin = 0,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Grimoire.Padding = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Grimoire.Padding end
			},
			Show = {
				name = 'Show',
				type = 'toggle',
				order = 4,
				set =
					function(info, input)
						self.db.global.Grimoire.Show = input;
						if input == true then
							if Whipped.Demons.GrimoireDemon.Talented then
								Frame_Grimoire:Show()
							else
								Frame_Grimoire:Hide()
							end
						else
							Frame_Grimoire:Hide()
						end
						Whipped:UpdateAllFrames()
					end,
				get =
					function(info)
						return self.db.global.Grimoire.Show
					end
			},
			Color = {
				name = 'Color',
				type = 'color',
				order = 5,
				set =
					function(info, r, g, b, a)
						self.db.global.Grimoire.Color.r = r;
						self.db.global.Grimoire.Color.g = g;
						self.db.global.Grimoire.Color.b = b;
						self.db.global.Grimoire.Color.a = a;
						Frame_Grimoire:SetStatusBarColor(r,g,b,a)
					end,
				get =
					function(info)
						local color = self.db.global.Grimoire.Color
						return color.r, color.g, color.b, color.a
					end
				
			},
			Color_Empowered = {
				name = 'Color Empowered',
				type = 'color',
				order = 6,
				set =
					function(info, r, g, b, a)
						self.db.global.Grimoire.Color_Empowered.r = r;
						self.db.global.Grimoire.Color_Empowered.g = g;
						self.db.global.Grimoire.Color_Empowered.b = b;
						self.db.global.Grimoire.Color_Empowered.a = a;
					end,
				get =
					function(info)
						local color = self.db.global.Grimoire.Color_Empowered
						return color.r, color.g, color.b, color.a
					end
				
			},
			Color_Cooldown = {
				name = 'Color Cooldown',
				type = 'color',
				order = 7,
				set =
					function(info, r, g, b, a)
						self.db.global.Grimoire.Color_Cooldown.r = r;
						self.db.global.Grimoire.Color_Cooldown.g = g;
						self.db.global.Grimoire.Color_Cooldown.b = b;
						self.db.global.Grimoire.Color_Cooldown.a = a;
					end,
				get =
					function(info)
						local color = self.db.global.Grimoire.Color_Cooldown
						return color.r, color.g, color.b, color.a
					end
				
			}
		}
	}
	self:AddCategory("Grimoire", Grimoire)
end

function Whipped:Grimoire_OnLoad()
	-- Statusbar
	local color = self.db.global.Grimoire.Color
	Frame_Grimoire:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
	Frame_Grimoire:SetMinMaxValues(0, 25)
	Frame_Grimoire:SetStatusBarColor(color.r, color.g, color.b, color.a)

	-- Icon Texture
	Frame_Grimoire_Icon.texture = Frame_Grimoire_Icon:CreateTexture(nil, "BACKGROUND")
	Frame_Grimoire_Icon.texture:SetTexture("Interface\\Icons\\warlock_grimoireofcommand")
	Frame_Grimoire_Icon.texture:SetAllPoints(true)
	
	-- Show / Hide
	if not self.db.global.Grimoire.Show then Frame_Grimoire:Hide(); Frame_Grimoire_Icon:Hide() end
end

function Whipped:Grimoire_OnUpdate()
	if GrimoireDemon.Summoned then -- Are Summoned or on Cooldown
		local t = GetTime()
		if GrimoireDemon.TTL > 0 then -- Living GrimoireDemon
			GrimoireDemon.TTL = GrimoireDemon.CD - t - 65
			Frame_Grimoire:SetValue(GrimoireDemon.TTL)
		else -- Dead GrimoireDemon
			if GrimoireDemon.TTL < 0 then -- Statement block only runs once every summon, Setup for cooldown
				GrimoireDemon.TTL = 0
				GrimoireDemon.Alive = 0
				local color = self.db.global.Grimoire.Color_Cooldown
				Frame_Grimoire:SetStatusBarColor(color.r, color.g, color.b, color.a)
			end
			local val = (65 - (GrimoireDemon.CD - t)) / 2.6 -- (CD(90) - TTL(25)) / TTL(25) = 2.6
			Frame_Grimoire:SetValue(val)
			if val > 25 then -- Statement block only runs once every cast, Check if off cooldown (reset the whole thing)
				GrimoireDemon.Dead = 0
				GrimoireDemon.GUID = 0
				GrimoireDemon.Summoned = false
				local color = self.db.global.Grimoire.Color
				Frame_Grimoire:SetStatusBarColor(color.r, color.g, color.b, color.a)
			end
		end
	end
end

function Whipped:Grimoire_UpdateFrames()
	local MasterFrame = self.db.global
	local SoulShards = MasterFrame.SoulShards
	local Imps = MasterFrame.Imps
	local Dreadstalkers = MasterFrame.Dreadstalkers
	local Cooldown = MasterFrame.Cooldown
	local Grimoire = MasterFrame.Grimoire
	local anchorID = MasterFrame.Anchor
	local anchor, Oanchor = Whipped.anchors[anchorID], Whipped:OppositeAnchor(anchorID)
	
	-- S = Statusbar, I = Icon
	local S_anchorFrame, S_anchorHelp, S_x, S_y, I_paddingX, I_paddingY = Whipped.Frame_Cooldown
	local temp = Cooldown
	if not Cooldown.Show then
		if not Dreadstalkers.Show then
			if not Imps.Show then
				if SoulShards.Show then
					S_anchorFrame = Whipped.Frame_SoulShard_1
					temp = SoulShards
				else
					S_anchorFrame = Whipped.MasterFrame
					temp = MasterFrame
				end
			else
				S_anchorFrame = Whipped.Frame_Imps_1
				temp = Imps
			end
		else
			S_anchorFrame = Whipped.Frame_Dreadstalkers
			temp = Dreadstalkers
		end
	end
	
	-- Logic
	if anchorID == 1 or anchorID == 2 then
		I_paddingY = 0
		if anchorID == 1 then
			I_paddingX = MasterFrame.PaddingX
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "TOP", MasterFrame.PaddingX, -(MasterFrame.PaddingY + Grimoire.Padding)
			else
				S_anchorHelp, S_x, S_y = "BOTTOM", 0, -(temp.Padding + Grimoire.Padding)
			end
		else
			I_paddingX = -MasterFrame.PaddingX
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "TOP", -MasterFrame.PaddingX, -(MasterFrame.PaddingY + Grimoire.Padding)
			else
				S_anchorHelp, S_x, S_y = "BOTTOM", 0, -(temp.Padding + Grimoire.Padding)
			end
		end
	else
		I_paddingX = 0
		if anchorID == 3 then
			I_paddingY = -MasterFrame.PaddingY
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "LEFT", (MasterFrame.PaddingX + Grimoire.Padding), -MasterFrame.PaddingY
			else
				S_anchorHelp, S_x, S_y = "RIGHT", (temp.Padding + Grimoire.Padding), 0
			end
		else
			I_paddingY = MasterFrame.PaddingY
			if temp == MasterFrame then
				S_anchorHelp, S_x, S_y = "LEFT", (MasterFrame.PaddingX + Grimoire.Padding), -MasterFrame.PaddingY
			else
				S_anchorHelp, S_x, S_y = "RIGHT", (temp.Padding + Grimoire.Padding), 0
			end
		end
	end
	
	-- Frame_Grimoire
	Frame_Grimoire:ClearAllPoints()
	if anchorID == 1 or anchorID == 2 then
		Frame_Grimoire:SetWidth((SoulShards.Width * 5 + SoulShards.Padding * 4))
		Frame_Grimoire:SetHeight(Grimoire.Thickness)
		Frame_Grimoire:SetPoint("TOP"..anchor, S_anchorFrame, S_anchorHelp..anchor, S_x, S_y)
		Frame_Grimoire:SetOrientation("HORIZONTAL")
	else
		Frame_Grimoire:SetWidth(Grimoire.Thickness)
		Frame_Grimoire:SetHeight((SoulShards.Height * 5 + SoulShards.Padding * 4))
		Frame_Grimoire:SetPoint(anchor.."LEFT", S_anchorFrame, anchor..S_anchorHelp, S_x, S_y)
		Frame_Grimoire:SetOrientation("VERTICAL")
	end
	
	-- Frame_Grimoire_Icon
	Frame_Grimoire_Icon:SetWidth(Grimoire.Thickness)
	Frame_Grimoire_Icon:SetHeight(Grimoire.Thickness)
	Frame_Grimoire_Icon:ClearAllPoints()
	Frame_Grimoire_Icon:SetPoint(anchor, Frame_Grimoire, Oanchor, I_paddingX, I_paddingY)
end

function Whipped:Grimoire_CLE(...)
	local event = select(2, ...)
	if event == "SPELL_CAST_SUCCESS" then
		if GrimoireDemon.Alive > 0 then
			local color = self.db.global.Grimoire.Color_Empowered
			Frame_Grimoire:SetStatusBarColor(color.r, color.g, color.b, color.a)
		end
	elseif event == "SPELL_AURA_REMOVED" then
		if GrimoireDemon.TTL > 0 then
			local color = self.db.global.Grimoire.Color
			Frame_Grimoire:SetStatusBarColor(color.r, color.g, color.b, color.a)
		end
	elseif event == "SPELL_SUMMON" then
		local SpellID = select(12, ...)
		if (SpellID == 111859 or SpellID == 111895 or SpellID == 111896 or SpellID == 111897 or SpellID == 111898)
			and bit.band(select(6,...), COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then
		-- Grimoire: Imp, Grimoire: Voidwalker, Grimoire: Succubus, Grimoire: Felhunter, Grimoire: Felguard
			GrimoireDemon.Summoned = true
			GrimoireDemon.CD = GetTime() + 90
			GrimoireDemon.TTL = GrimoireDemon.CD - 65
			GrimoireDemon.Alive = 1
			GrimoireDemon.DE = 1
			GrimoireDemon.Name = select(9, ...)
			GrimoireDemon.GUID = select(8, ...)
		end
	elseif event == "UNIT_DIED" then
		if select(8,...) == Demons.GrimoireDemon.GUID then -- Grimoire: Felguard
			GrimoireDemon.Alive = 0
			GrimoireDemon.TTL = 0
			local color = self.db.global.Grimoire.Color
			Frame_Grimoire:SetStatusBarColor(color.r, color.g, color.b, color.a)
		end
	end
end

function Whipped:Grimoire_UpdateTalents()
	local id,_,_,b = GetTalentInfo(6,2,1)
	if id == 19294 and b then
		GrimoireDemon.Talented = true
		if self.db.global.Grimoire.Show then
			Frame_Grimoire:Show()
		end
	else
		GrimoireDemon.Talented = false
		Frame_Grimoire:Hide()
	end
end